#!/bin/sh
# sl v1.0.3  (c) 25.3.2010 by Andreas Ley  (u) 30.6.2010
# Search LDAP

usage()
{
	echo "Usage: `basename $0` [-f|-k] [-u|-U|-m|-g|-G|-H|-S|-M] [-1] [key [...]]" >&2
#	echo "-u  search uni-karlsruhe.de (default)" >&2
	echo "-f  search fzk.de" >&2
	echo "-k  search kit.edu" >&2
	echo "-u  search user" >&2
	echo "-U  search user id" >&2
	echo "-m  search mail address" >&2
	echo "-g  search group" >&2
	echo "-G  search group id" >&2
	echo "-H  search home directory" >&2
	echo "-S  search sys directory" >&2
	echo "-M  search mail routing" >&2
	echo "-1  output latin1 (default quoted utf8)" >&2
	echo "-8  output utf8 (default quoted utf8)" >&2
	exit 1
}

set -- `getopt hxvfkuUmMgGHS1 $*` || usage

trace=false
verbose=false
search=raw
decode=false
domain=uni-karlsruhe.de
while :; do
	case $1 in
		-h)	sed '1d;s/^# *//;/^$/q' $0; usage;;
		-x)	set -x; trace=true; shift;;
		-v)	verbose=true; shift;;
#		-u)	domain=uni-karlsruhe.de; shift;;
		-f)	domain=fzk.de; shift;;
		-k)	domain=kit.edu; shift;;
		-u)	search=user; shift;;
		-U)	search=uid; shift;;
		-m)	search=mail; shift;;
		-M)	search=route; shift;;
		-g)	search=group; shift;;
		-G)	search=gid; shift;;
		-H)	search=home; shift;;
		-S)	search=sys; shift;;
		-1)	decode=true; recode="u8..l1"; shift;;
		-8)	decode=true; shift;;
		--)	shift; break;;
	esac
done

case ${domain} in
	uni-karlsruhe.de)
		access='-h rzldap01.rz.uni-karlsruhe.de'
		case ${search} in
			user)	base="ou=people,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="uid"
				;;
			uid)	base="ou=people,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="uidNumber"
				;;
			group)	base="ou=groups,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="cn"
				;;
			gid)	base="ou=groups,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="gidNumber"
				;;
			home)	base="ou=home.ws,ou=autofs,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="cn"
				;;
			sys)	base="ou=auto.home.sys,ou=autofs,dc=rz,dc=uni-karlsruhe,dc=de"
				attr="cn"
				;;
			route)	base="ou=mailrouting,dc=mail,dc=uni-karlsruhe,dc=de"
				filter="objectClass=rzMailDomain"
				attr="rzMailDomainName"
				#attr="rzMailDomainAliasName"
				;;
		esac
		;;
	fzk.de)
		case ${search} in
			user)	base="DC=fzk,DC=de"
				filter="objectClass=user"
				attr="sAMAccountName"
				;;
			uid)	base="DC=fzk,DC=de"
				filter="objectClass=user"
				attr="msSFU30UidNumber"
				;;
			mail)	base="DC=fzk,DC=de"
				filter="objectClass=user"
				attr="mail"
				;;
			group)	base="OU=Groups,OU=Unix,DC=ka,DC=fzk,DC=de"
				filter="objectClass=group"
				attr="sAMAccountName"
				;;
			gid)	base="OU=Groups,OU=Unix,DC=ka,DC=fzk,DC=de"
				filter="objectClass=group"
				attr="msSFU30GidNumber"
				;;
		esac
		;;
	kit.edu)
		access='-h kit-dc-01.kit.edu -D CN=padl,CN=Users,dc=kit,dc=edu -w rw6344tt'
		case ${search} in
			user)	base="dc=kit,dc=edu"
				filter="objectClass=user"
				attr="sAMAccountName"
				;;
			uid)	base="dc=kit,dc=edu"
				filter="objectClass=user"
				attr="msSFU30UidNumber"
				;;
			mail)	base="dc=kit,dc=edu"
				filter="objectClass=user"
				attr="mail"
				;;
			group)	base="dc=kit,dc=edu"
				filter="objectClass=group"
				attr="sAMAccountName"
				;;
			gid)	base="dc=kit,dc=edu"
				filter="objectClass=group"
				attr="msSFU30GidNumber"
				;;
		esac
		;;
esac

if test -s $HOME/.access/ldap/${domain}; then
	access="`cat $HOME/.access/ldap/${domain}`"
elif test -z "${access}"; then
	echo "Can't find $HOME/.access/ldap/${domain}" >&2
	exit 1
fi

ldapsearch='/usr/segment/bin/ldapsearch'
test -x /usr/bin/ldapsearch && ldapsearch='/usr/bin/ldapsearch'

if test "${search}" = "raw"; then
	if test -n "${recode}"; then
		${ldapsearch} -x ${access} ${1+"$@"} | recode "${recode}"
		exit $?
	else
		exec ${ldapsearch} -x ${access} ${1+"$@"}
	fi
fi

if test -z "${base}"; then
	echo "Unsupported combination" >&2
	usage
fi

for key; do
	if test -n "${recode}"; then
		${ldapsearch} -x ${access} -b ${base} "${filter+(&(${filter})}(${attr}=${key})${filter+)}" | perl -pe 'if (/\\[0-9A-F][0-9A-F]/) { while (/\\([0-9A-F][0-9A-F])/) { $d .= $`.chr(hex($1)); $_ = $'"'"'; } $_ = $d.$_; }' | recode "${recode}"
	elif test -n "${decode}"; then
		${ldapsearch} -x ${access} -b ${base} "${filter+(&(${filter})}(${attr}=${key})${filter+)}" | perl -pe 'if (/\\[0-9A-F][0-9A-F]/) { while (/\\([0-9A-F][0-9A-F])/) { $d .= $`.chr(hex($1)); $_ = $'"'"'; } $_ = $d.$_; }'
	else
		${ldapsearch} -x ${access} -b ${base} "${filter+(&(${filter})}(${attr}=${key})${filter+)}"
	fi
done

exit 0
